﻿using System;
using System.Collections.Generic;
using System.Collections.Specialized;
using System.IO;
using System.Net;
using System.Net.Security;
using System.Security.Cryptography.X509Certificates;
using System.Text;
using System.Text.RegularExpressions;
using System.Web;
using Pz.API.QH360.SDK.Common;
using Pz.API.QH360.SDK.Model;

namespace Pz.API.QH360.SDK.API
{
    /// <summary>
    /// Author2.0验证api
    /// </summary>
    public class APIAuthorize
    {
        /// <summary>
        /// AppKey
        /// </summary>
        public string AppKey { get; set; }

        /// <summary>
        /// AppSecret
        /// </summary>
        public string AppSecret { get; set; }

        private const string apiHost = "https://openapi.360.cn";
        private const string apiCallBackUrl = "oob";
        private const string apiScope = "basic";

        /// <summary>
        /// 通过RefreshToken来刷新获取AccessToken
        /// </summary>
        /// <param name="refreshToken"></param>
        /// <returns>MAccessToken</returns>
        public MAccessToken GetAccessTokenByRefreshToken(string refreshToken)
        {
            if (string.IsNullOrEmpty(AppKey))
                throw new Exception("AppKey  Can't be empty");
            if (string.IsNullOrEmpty(AppSecret))
                throw new Exception("AppSecret  Can't be empty");
            MAccessToken accessToken = null;
            string requestRefreshTokenUrl = "{0}access_token?grant_type=refresh_token&client_id={1}&client_secret={2}&refresh_token={3}";
            string url = string.Format(requestRefreshTokenUrl, MyConfig.OAuthHostUrl, AppKey, AppSecret, refreshToken);
            accessToken = null;
            string resData = getResponse(url);
            accessToken = Json.Deserialize<MAccessToken>(resData);
            return accessToken;
        }

        /// <summary>
        /// 验证用户名密码是否有效,若有效返回它的code信息,若无效返回错误代码
        /// 返回0,验证成功;返回4010206用户名密码不匹配,其他,网络或其他异常;
        /// </summary>
        /// <param name="username"></param>
        /// <param name="password"></param>
        /// <param name="code"></param>
        /// <returns></returns>
        public int ValidationUser(string username, string password, string valcode, CookieCollection aCookies, ref string code, ref string valcodeurl)
        {
            if (string.IsNullOrEmpty(AppKey))
                throw new Exception("AppKey  Can't be empty");
            if (string.IsNullOrEmpty(AppSecret))
                throw new Exception("AppSecret  Can't be empty");
            code = string.Empty;
            int result = 0;
            List<MyPostParameter> liPar = new List<MyPostParameter>();
            liPar.Add(new MyPostParameter("client_id", AppKey));
            liPar.Add(new MyPostParameter("response_type", "code"));
            liPar.Add(new MyPostParameter("redirect_uri", apiCallBackUrl));
            liPar.Add(new MyPostParameter("state", ""));
            liPar.Add(new MyPostParameter("scope", apiScope));
            liPar.Add(new MyPostParameter("display", "dianjing"));
            liPar.Add(new MyPostParameter("mid", ""));
            liPar.Add(new MyPostParameter("version", ""));
            liPar.Add(new MyPostParameter("DChannel", ""));
            if (!string.IsNullOrEmpty(valcode))
                liPar.Add(new MyPostParameter("captcha", valcode));
            //liPar.Add(new MyPostParameter("isAuthorize", "on")); //勾选了360开放平台用户服务协议
            liPar.Add(new MyPostParameter("authorize", "%09"));  //必需的加 ,否则一直授权不了

            liPar.Add(new MyPostParameter("type", "qihoo"));
            liPar.Add(new MyPostParameter("username", username));
            liPar.Add(new MyPostParameter("password", password));

            string requestUrl = string.Format("{0}authorize", MyConfig.OAuthHostUrl);
            byte[] postData = getPostData(liPar);
            HttpWebRequest aHttpWebRequest = null;
            try
            {
                if (requestUrl.StartsWith("https", StringComparison.OrdinalIgnoreCase))
                    ServicePointManager.ServerCertificateValidationCallback = new RemoteCertificateValidationCallback(CheckValidationResult);
                aHttpWebRequest = WebRequest.Create(requestUrl) as HttpWebRequest;
                aHttpWebRequest.Method = "POST";
                aHttpWebRequest.ProtocolVersion = HttpVersion.Version10;
                aHttpWebRequest.UserAgent = "curl/7.28.1";
                aHttpWebRequest.Accept = "*/*";
                aHttpWebRequest.KeepAlive = false;
                aHttpWebRequest.ContentType = "application/x-www-form-urlencoded";
                aHttpWebRequest.Proxy = null;
                aHttpWebRequest.CookieContainer = new CookieContainer();
                if (null != aCookies)
                    foreach (Cookie c in aCookies)
                        aHttpWebRequest.CookieContainer.Add(new Cookie() { Name = c.Name, Value = c.Value, Domain = "openapi.360.cn", Path = c.Path });
                if (null == postData)
                    postData = new byte[0];
                aHttpWebRequest.ContentLength = postData.Length;
                if (postData.Length > 0)
                {
                    using (Stream aStream = aHttpWebRequest.GetRequestStream())
                    {
                        aStream.Write(postData, 0, postData.Length);
                    }
                }
                using (HttpWebResponse aHttpWebResponse = (HttpWebResponse)aHttpWebRequest.GetResponse())
                {
                    aCookies.Add(aHttpWebResponse.Cookies);
                    Uri uri = aHttpWebResponse.ResponseUri;
                    NameValueCollection col = HttpUtility.ParseQueryString(uri.Query, Encoding.UTF8);
                    using (StreamReader reader = new StreamReader(aHttpWebResponse.GetResponseStream(), Encoding.UTF8))
                    {
                        string text = reader.ReadToEnd();

                        Regex r = new Regex("<div[^>]*class=\"err\">.*?用户名和密码不匹配.*?</div>", RegexOptions.IgnoreCase | RegexOptions.Singleline);
                        Match m = r.Match(text);
                        if (m.Success)
                            result = 4010206;
                        else
                        {
                            r = new Regex("<img[^>]*id=\"captchaImage\"[^>]*src=\"(?<codeurl>[^\"]*)\">", RegexOptions.IgnoreCase | RegexOptions.Singleline);
                            m = r.Match(text);
                            if (m.Success)
                                valcodeurl = m.Groups["codeurl"].ToString();
                        }
                    }
                    if (col == null)
                        throw new WebException("获取Code失败");
                    code = col["code"];
                }
            }
            catch (WebException ex)
            {
                string aHtml = "";
                if (null != ex.Response)
                {
                    using (HttpWebResponse aHttpWebResponse = (HttpWebResponse)ex.Response)
                    {
                        using (StreamReader reader = new StreamReader(aHttpWebResponse.GetResponseStream(), MyConfig.MyEncoding))
                            aHtml = reader.ReadToEnd();
                        OAuthFailures oAuthFailures = Json.Deserialize<OAuthFailures>(aHtml);
                        result = int.Parse(oAuthFailures.error_code);
                    }
                }
                if (string.IsNullOrEmpty(aHtml))
                    throw ex;
            }
            finally
            {
                if (aHttpWebRequest != null)
                {
                    aHttpWebRequest.Abort();
                    aHttpWebRequest = null;
                }
            }
            return result;
        }

        /// <summary>
        /// 获取登录的验证码
        /// </summary>
        /// <param name="cookies"></param>
        /// <returns></returns>
        public string GetHtmlValidation(ref CookieCollection cookies)
        {
            if (string.IsNullOrEmpty(AppKey))
                throw new Exception("AppKey  Can't be empty");
            string requestUrl = string.Format("{0}authorize?client_id={1}&response_type=code&scope=basic&display=dianjing", MyConfig.OAuthHostUrl, AppKey);
            HttpWebRequest aHttpWebRequest = null;
            try
            {
                if (requestUrl.StartsWith("https", StringComparison.OrdinalIgnoreCase))
                    ServicePointManager.ServerCertificateValidationCallback = new RemoteCertificateValidationCallback(CheckValidationResult);
                aHttpWebRequest = WebRequest.Create(requestUrl) as HttpWebRequest;
                aHttpWebRequest.Method = "GET";
                aHttpWebRequest.ProtocolVersion = HttpVersion.Version10;
                aHttpWebRequest.UserAgent = "curl/7.28.1";
                aHttpWebRequest.Accept = "*/*";
                aHttpWebRequest.KeepAlive = false;
                aHttpWebRequest.ContentType = "application/x-www-form-urlencoded";
                aHttpWebRequest.Proxy = null;
                aHttpWebRequest.CookieContainer = new CookieContainer();
                using (HttpWebResponse aHttpWebResponse = (HttpWebResponse)aHttpWebRequest.GetResponse())
                {
                    cookies.Add(aHttpWebResponse.Cookies);
                    using (StreamReader reader = new StreamReader(aHttpWebResponse.GetResponseStream(), Encoding.UTF8))
                    {
                        string text = reader.ReadToEnd();
                        Regex r = new Regex("<div[^>]*class=\"err\">.*?用户名和密码不匹配.*?</div>", RegexOptions.IgnoreCase | RegexOptions.Singleline);
                        Match m = r.Match(text);
                        r = new Regex("<img[^>]*id=\"captchaImage\"[^>]*src=\"(?<codeurl>[^\"]*)\">", RegexOptions.IgnoreCase | RegexOptions.Singleline);
                        m = r.Match(text);
                        if (m.Success)
                            return m.Groups["codeurl"].ToString();
                    }
                }
                return string.Empty;
            }
            catch { }
            finally
            {
                if (null != aHttpWebRequest)
                    aHttpWebRequest.Abort();
            }
            return string.Empty;
        }

        /// <summary>
        /// 获取360的验证码
        /// </summary>
        /// <param name="image">验证码图片内容</param>
        /// <returns></returns>
        public void GetValue(string url, ref CookieCollection cookies, out byte[] image)
        {
            image = null;
            HttpWebRequest aHttpWebRequest = null;
            HttpWebResponse aHttpWebResponse = null;
            try
            {
                aHttpWebRequest = (HttpWebRequest)HttpWebRequest.Create(url);
                aHttpWebRequest.Proxy = null;
                aHttpWebRequest.CookieContainer = new CookieContainer();
                aHttpWebRequest.CookieContainer.Add(cookies);
                using (aHttpWebResponse = (HttpWebResponse)aHttpWebRequest.GetResponse())
                {
                    cookies.Add(aHttpWebResponse.Cookies);
                    using (Stream s = aHttpWebResponse.GetResponseStream())
                    {
                        using (var memstream = new MemoryStream())
                        {
                            var buffer = new byte[512];
                            var bytesRead = default(int);
                            while ((bytesRead = s.Read(buffer, 0, buffer.Length)) > 0)
                                memstream.Write(buffer, 0, bytesRead);
                            image = memstream.ToArray();
                        }
                    }
                }
            }
            catch { }
            finally
            {
                aHttpWebResponse = null;
                aHttpWebRequest = null;
            }
        }

        /// <summary>
        /// 通过code获取AccessToken 
        /// </summary>
        /// <param name="code">code</param>
        /// <returns>MAccessToken</returns>
        public MAccessToken GetAccessTokenByCode(string code)
        {
            if (string.IsNullOrEmpty(AppKey))
                throw new Exception("AppKey  Can't be empty");
            if (string.IsNullOrEmpty(AppSecret))
                throw new Exception("AppSecret  Can't be empty");
            string redirectUrl = string.Format("{0}access_token?grant_type=authorization_code&code={1}&client_id={2}&client_secret={3}&redirect_uri={4}", MyConfig.OAuthHostUrl, code, AppKey, AppSecret, apiScope);
            string ahtml = getResponse(redirectUrl);
            if (string.IsNullOrEmpty(ahtml))
                return null;
            string accessTokenStr = ahtml;
            MAccessToken accessToken = null;
            accessToken = Json.Deserialize<MAccessToken>(accessTokenStr);
            return accessToken;
        }

        /// <summary>
        /// 获取360线上的账户信息
        /// </summary>
        /// <param name="accessToken">accessToken</param>
        /// <returns></returns>
        public UserMe GetUserMe(string accessToken)
        {
            string url = string.Format("{0}/user/me.json?access_token={1}", apiHost, accessToken);
            string userInfoJson = getResponse(url);
            UserMe userMe = Json.Deserialize<UserMe>(userInfoJson);
            return userMe;
        }

        /// <summary>
        /// 检测当前AccessToken是否过期或可用
        /// </summary>
        /// <param name="accessToken"></param>
        /// <returns></returns>
        public bool CheckTokenIsEnable(string accessToken)
        {
            if (string.IsNullOrEmpty(AppKey))
                throw new Exception("AppKey  Can't be empty");
            bool result = false;
            APIAccount aPIAccount = new APIAccount();
            aPIAccount.AuthHeader = new PostHeaders()
            {
                apiKey = AppKey,
                accessToken = accessToken
            };
            MAccountInfo accountInfo = null;
            Result res = aPIAccount.getInfo(ref accountInfo);
            if (!res.IsAccessTokenExpires)
                result = true;
            return result;
        }

        /// <summary>
        /// 根据url获取返回的内容
        /// </summary>
        /// <param name="url"></param>
        /// <returns></returns>
        private string getResponse(string url)
        {
            string aHtml = string.Empty;
            HttpWebRequest aHttpWebRequest = null;
            try
            {
                if (url.StartsWith("https", StringComparison.OrdinalIgnoreCase))
                    ServicePointManager.ServerCertificateValidationCallback = new RemoteCertificateValidationCallback(CheckValidationResult);
                aHttpWebRequest = WebRequest.Create(url) as HttpWebRequest;
                aHttpWebRequest.Method = "GET";
                aHttpWebRequest.ProtocolVersion = HttpVersion.Version10;
                aHttpWebRequest.UserAgent = "curl/7.28.1";
                aHttpWebRequest.Accept = "*/*";
                aHttpWebRequest.KeepAlive = false;
                aHttpWebRequest.ContentType = "application/x-www-form-urlencoded";
                aHttpWebRequest.Proxy = null;

                using (HttpWebResponse aHttpWebResponse = (HttpWebResponse)aHttpWebRequest.GetResponse())
                {
                    using (StreamReader reader = new StreamReader(aHttpWebResponse.GetResponseStream(), MyConfig.MyEncoding))
                        aHtml = reader.ReadToEnd();
                }
            }
            catch (WebException ex)
            {
                if (null != ex.Response)
                {
                    using (HttpWebResponse aHttpWebResponse = (HttpWebResponse)ex.Response)
                    {
                        using (StreamReader reader = new StreamReader(aHttpWebResponse.GetResponseStream(), MyConfig.MyEncoding))
                            aHtml = reader.ReadToEnd();
                    }
                }
                if (string.IsNullOrEmpty(aHtml))
                    throw ex;
            }
            finally
            {
                if (aHttpWebRequest != null)
                {
                    aHttpWebRequest.Abort();
                    aHttpWebRequest = null;
                }
            }
            return aHtml;
        }
        /// <summary>
        /// 获取提交数据
        /// </summary>
        /// <param name="parameters"></param>
        /// <returns></returns>
        private byte[] getPostData(List<MyPostParameter> parameters)
        {
            string data = "format=" + MyConfig.MyFormat.ToString();
            if (null != parameters)
            {
                foreach (MyPostParameter p in parameters)
                {
                    string name = p.Name;
                    string value = "";
                    object objValue = p.Value;
                    if (objValue != null)
                    {
                        value = objValue.ToString();
                        value = HttpUtility.UrlEncode(value, MyConfig.PostEncoding);
                    }
                    data += "&" + name + "=" + value;
                }
            }
            return MyConfig.PostEncoding.GetBytes(data);
        }

        private bool CheckValidationResult(object sender, X509Certificate certificate, X509Chain chain, SslPolicyErrors errors)
        {
            //总是接受          
            return true;
        }

        /// <summary>
        /// Author2.0验证api返回的错误信息
        /// </summary>
        [Serializable]
        public class OAuthFailures
        {
            /// <summary>
            /// 错误代码
            /// </summary>
            public string error_code { get; set; }
            /// <summary>
            /// 错误消息
            /// </summary>
            public string error { get; set; }
        }
    }
}
