﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Pz.API.QH360.SDK.Enum;
using Pz.API.QH360.SDK.Model;
using Pz.API.QH360.SDK.Common;

namespace Pz.API.QH360.SDK.API
{
    /// <summary>
    /// 报表服务
    /// </summary>
    public class APIReport : APIParent
    {
        /// <summary>
        /// 设置服务的类型
        /// </summary>
        internal override ServiceType ServiceType
        {
            get { return ServiceType.report; }
        }

        #region 服务接口类

        /// <summary>
        /// 关键词报告记录数量
        /// </summary>
        /// <param name="startDate">起始日期，最大跨度90天 </param>
        /// <param name="level">物料层级，account代表账户级别，plan代表计划级别，group代表组级别</param>
        /// <param name="idList">如果是账户不填，其他层级需要对应的计划id或组id，使用json格式，如[1000001,20000001] </param>
        /// <param name="totalNumber">返回的总记录数</param>
        /// <param name="totalPage">返回的总页数</param>
        /// <returns></returns>
        public Result keywordCount(DateTime startDate, string level, long[] idList, ref int totalNumber, ref int totalPage)
        {
            List<MyPostParameter> pars = new List<MyPostParameter>();
            pars.Add(new MyPostParameter("startDate", string.Format(new CustomDateFormatter(), "{0:S}", startDate)));
            pars.Add(new MyPostParameter("level", level));
            if (!level.Equals("account"))
            {
                if (idList == null)
                    idList = new long[0];
                pars.Add(new MyPostParameter("idList", Json.SerializeObject(idList)));
            }
            Result res = new Result();
            MyHttpRequest req = new MyHttpRequest(this);

            req.methodType = MethodType.keywordCount;
            req.postResponse(pars);
            req.GetRes(ref res);
            switch (req.status)
            {
                case ResultStatus.Succeed:
                    ReportCount apiRes = Json.Deserialize<ReportCount>(req.data);
                    if (null == apiRes)
                        break;
                    res.Failures = apiRes.failures;
                    totalNumber = apiRes.totalNumber;
                    totalPage = apiRes.totalPage;
                    break;
                case ResultStatus.Fail:
                    break;
                case ResultStatus.DataIsEmpty:
                    break;
            }
            return res;
        }
        /// <summary>
        /// 生成关键词报告 
        /// </summary>
        /// <param name="startDate">起始日期，最大跨度90天 </param>
        /// <param name="level">物料层级，account代表账户级别，plan代表计划级别，group代表组级别</param>
        /// <param name="idList">如果是账户不填，其他层级需要对应的计划id或组id，使用json格式，如[1000001,20000001] </param>
        /// <param name="page">页号</param>
        /// <param name="reportList"></param>
        /// <returns></returns>
        public Result keyword(DateTime startDate, string level, long[] idList, int page,ref List<MReportKeyword> reportList)
        {
            List<MyPostParameter> pars = new List<MyPostParameter>();
            pars.Add(new MyPostParameter("startDate", string.Format(new CustomDateFormatter(), "{0:S}", startDate)));
            pars.Add(new MyPostParameter("level", level));
            if (!level.Equals("account"))
            {
                if (idList == null)
                    idList = new long[0];
                pars.Add(new MyPostParameter("idList", Json.SerializeObject(idList)));
            }
            pars.Add(new MyPostParameter("page", page));
            Result res = new Result();
            MyHttpRequest req = new MyHttpRequest(this);

            req.methodType = MethodType.keyword;
            req.postResponse(pars);
            req.GetRes(ref res);
            switch (req.status)
            {
                case ResultStatus.Succeed:
                    KeywordList apiRes = Json.Deserialize<KeywordList>(req.data);
                    if (null == apiRes)
                        break;
                    res.Failures = apiRes.failures;
                    reportList = apiRes.keywordList;
                    break;
                case ResultStatus.Fail:
                    break;
                case ResultStatus.DataIsEmpty:
                    break;
            }
            return res;
        }

        /// <summary>
        /// 创意报告记录数量
        /// </summary>
        /// <param name="startDate">起始日期，最大跨度90天 </param>
        /// <param name="level">物料层级，account代表账户级别，plan代表计划级别，group代表组级别</param>
        /// <param name="idList">如果是账户不填，其他层级需要对应的计划id或组id，使用json格式，如[1000001,20000001] </param>
        /// <param name="totalNumber">返回的总记录数</param>
        /// <param name="totalPage">返回的总页数</param>
        /// <returns></returns>
        public Result creativeCount(DateTime startDate, string level, long[] idList, ref int totalNumber, ref int totalPage)
        {
            List<MyPostParameter> pars = new List<MyPostParameter>();
            pars.Add(new MyPostParameter("startDate", string.Format(new CustomDateFormatter(), "{0:S}", startDate)));
            pars.Add(new MyPostParameter("level", level));
            if (!level.Equals("account"))
            {
                if (idList == null)
                    idList = new long[0];
                pars.Add(new MyPostParameter("idList", Json.SerializeObject(idList)));
            }
            Result res = new Result();
            MyHttpRequest req = new MyHttpRequest(this);

            req.methodType = MethodType.creativeCount;
            req.postResponse(pars);
            req.GetRes(ref res);
            switch (req.status)
            {
                case ResultStatus.Succeed:
                    ReportCount apiRes = Json.Deserialize<ReportCount>(req.data);
                    if (null == apiRes)
                        break;
                    res.Failures = apiRes.failures;
                    totalNumber = apiRes.totalNumber;
                    totalPage = apiRes.totalPage;
                    break;
                case ResultStatus.Fail:
                    break;
                case ResultStatus.DataIsEmpty:
                    break;
            }
            return res;
        }

        /// <summary>
        /// 生成创意报告 
        /// </summary>
        /// <param name="startDate">起始日期，最大跨度90天 </param>
        /// <param name="level">物料层级，account代表账户级别，plan代表计划级别，group代表组级别</param>
        /// <param name="idList">如果是账户不填，其他层级需要对应的计划id或组id，使用json格式，如[1000001,20000001] </param>
        /// <param name="page">页号</param>
        /// <param name="reportList"></param>
        /// <returns></returns>
        public Result creative(DateTime startDate, string level, long[] idList, int page,ref List<MReportCreative> reportList)
        {
            List<MyPostParameter> pars = new List<MyPostParameter>();
            pars.Add(new MyPostParameter("startDate", string.Format(new CustomDateFormatter(), "{0:S}", startDate)));
            pars.Add(new MyPostParameter("level", level));
            if (!level.Equals("account"))
            {
                if (idList == null)
                    idList = new long[0];
                pars.Add(new MyPostParameter("idList", Json.SerializeObject(idList)));
            }
            pars.Add(new MyPostParameter("page", page));
            Result res = new Result();
            MyHttpRequest req = new MyHttpRequest(this);

            req.methodType = MethodType.creative;
            req.postResponse(pars);
            req.GetRes(ref res);
            switch (req.status)
            {
                case ResultStatus.Succeed:
                    CreativeList apiRes = Json.Deserialize<CreativeList>(req.data);
                    if (null == apiRes)
                        break;
                    res.Failures = apiRes.failures;
                    reportList = apiRes.creativeList;
                    break;
                case ResultStatus.Fail:
                    break;
                case ResultStatus.DataIsEmpty:
                    break;
            }
            return res;
        }


        /// <summary>
        /// 地域报告记录数量
        /// </summary>
        /// <param name="startDate">起始日期，最大跨度90天 </param>
        /// <param name="level">物料层级，account代表账户级别，plan代表计划级别，group代表组级别</param>
        /// <param name="idList">如果是账户不填，其他层级需要对应的计划id或组id，使用json格式，如[1000001,20000001] </param>
        /// <param name="region">默 认 全 国 ， 其 他 填 写 地 域id ，json 格 式 ， 如[1000001,20000001]</param>
        /// <param name="totalNumber">返回的总记录数</param>
        /// <param name="totalPage">返回的总页数</param>
        /// <returns></returns>
        public Result regionCount(DateTime startDate, string level, long[] idList, long[] region, ref int totalNumber, ref int totalPage)
        {
            List<MyPostParameter> pars = new List<MyPostParameter>();
            pars.Add(new MyPostParameter("startDate", string.Format(new CustomDateFormatter(), "{0:S}", startDate)));
            pars.Add(new MyPostParameter("level", level));
            if (!level.Equals("account"))
            {
                if (idList == null)
                    idList = new long[0];
                pars.Add(new MyPostParameter("idList", Json.SerializeObject(idList)));
            }
            if (region != null && region.Length>0)
                pars.Add(new MyPostParameter("region", Json.SerializeObject(region)));
            Result res = new Result();
            MyHttpRequest req = new MyHttpRequest(this);

            req.methodType = MethodType.regionCount;
            req.postResponse(pars);
            req.GetRes(ref res);
            switch (req.status)
            {
                case ResultStatus.Succeed:
                    ReportCount apiRes = Json.Deserialize<ReportCount>(req.data);
                    if (null == apiRes)
                        break;
                    res.Failures = apiRes.failures;
                    totalNumber = apiRes.totalNumber;
                    totalPage = apiRes.totalPage;
                    break;
                case ResultStatus.Fail:
                    break;
                case ResultStatus.DataIsEmpty:
                    break;
            }
            return res;
        }

        /// <summary>
        ///生成地域报告 
        /// </summary>
        /// <param name="startDate">起始日期，最大跨度90天 </param>
        /// <param name="level">物料层级，account代表账户级别，plan代表计划级别，group代表组级别</param>
        /// <param name="idList">如果是账户不填，其他层级需要对应的计划id或组id，使用json格式，如[1000001,20000001] </param>
         /// <param name="region">默 认 全 国 ， 其 他 填 写 地 域id ，json 格 式 ， 如[1000001,20000001]</param>
        /// <param name="page">页号</param>
        /// <param name="reportList"></param>
        /// <returns></returns>
        public Result region(DateTime startDate, string level, long[] idList,long[] region, int page,ref List<MReportRegion> reportList)
        {
            List<MyPostParameter> pars = new List<MyPostParameter>();
            pars.Add(new MyPostParameter("startDate", string.Format(new CustomDateFormatter(), "{0:S}", startDate)));
            pars.Add(new MyPostParameter("level", level));
            if (!level.Equals("account"))
            {
                if (idList == null)
                    idList = new long[0];
                pars.Add(new MyPostParameter("idList", Json.SerializeObject(idList)));
            }
            if (region != null && region.Length > 0)
                pars.Add(new MyPostParameter("region", Json.SerializeObject(region)));
            pars.Add(new MyPostParameter("page", page));
            Result res = new Result();
            MyHttpRequest req = new MyHttpRequest(this);

            req.methodType = MethodType.region;
            req.postResponse(pars);
            req.GetRes(ref res);
            switch (req.status)
            {
                case ResultStatus.Succeed:
                    RegionList apiRes = Json.Deserialize<RegionList>(req.data);
                    if (null == apiRes)
                        break;
                    res.Failures = apiRes.failures;
                    reportList = apiRes.regionList;
                    break;
                case ResultStatus.Fail:
                    break;
                case ResultStatus.DataIsEmpty:
                    break;
            }
            return res;
        }

        #endregion

        #region Json解析辅助类
        /// <summary>
        /// 计划列表
        /// </summary>
        [Serializable]
        private class ReportCount : ApiResult
        {
            /// <summary>
            /// 总数量
            /// </summary>
            public int totalNumber { get; set; }

            /// <summary>
            /// 总页数
            /// </summary>
            public int totalPage { get; set; }
        }


        [Serializable]
        private class KeywordList : ApiResult
        {
            public List<MReportKeyword> keywordList { get; set; }
        }

        [Serializable]
        private class CreativeList : ApiResult
        {
            public List<MReportCreative> creativeList { get; set; }
        }

        [Serializable]
        private class RegionList : ApiResult
        {
            public List<MReportRegion> regionList { get; set; }
        }
        #endregion
    }
}
