﻿using System;
using System.Collections.Generic;
using System.Text;
using Pz.API.QH360.SDK.Model;
using System.Threading;
using Pz.API.QH360.SDK.API;

namespace Pz.API.QH360.SDK.Common
{
    /// <summary>
    /// 用于多线程下载物料信息
    /// </summary>
    /// <typeparam name="T"></typeparam>
    public abstract class AsyDownLoadBase<T>
    {
        public APIParent ApiService { get; set; }
        public List<Result> ResultList { get;private set; }
        public List<T> Models { get; set; }
        public List<long[]> Parameters { get; set; }
        public  WorkReportsProgressDelegate ReportsProgressHandle;

        private List<Thread> threadPool { get; set; }
        private Queue<long[]> queueOfWork { get; set; }
        private ManualResetEvent manualResetEvent4Stop = new ManualResetEvent(false);
        private object lockerOfQueue = new object();
        private int maxThreadCount = MyConfig.MultiThreadCount; //可以开启的最大线程数量


        private  void Initialize()
        {
            ResultList = new List<Result>();
            Models = new List<T>();
            threadPool = new List<Thread>();
            queueOfWork = new Queue<long[]>();
            //首先把参数数据添加到队列里面
            foreach (long[] ids in Parameters)
            {
                queueOfWork.Enqueue(ids);
            }
            //初始化线程
            if (queueOfWork.Count <= maxThreadCount)
                maxThreadCount = queueOfWork.Count;  //队列里面的数量比较小,线程数就不用那么大了
            for (int i = 0; i < maxThreadCount; i++)
            {
                Thread thread = new Thread(new ThreadStart(DoWork));
                threadPool.Add(thread);
            }
        }

        public virtual void Start()
        {
            Initialize();
            threadPool.ForEach(_=>_.Start());
            manualResetEvent4Stop.WaitOne();
            //等待子线程彻底结束
            threadPool.ForEach(_=>_.Join());
            manualResetEvent4Stop.Reset();
        }
        private  void DoWork()
        {
            while (true)
            {
                #region Get Work
                long[] ids = new long[0];
                bool haveWork = false;
                lock (this.lockerOfQueue)
                {
                    if (this.queueOfWork.Count > 0)
                    {
                        ids = this.queueOfWork.Dequeue();
                        haveWork = true;
                    }
                    //360限制为每秒的最大请求数5个并发,假如并发数过多,下载下来的数据可能会紊乱,这个已经测试过了
                    //为了把它限制在5个以内,故让线程等待200毫秒,这样数据不会错了
                        manualResetEvent4Stop.WaitOne(300);
                }
                #endregion

                if (haveWork)
                {
                    Process(ids);
                }
                else
                {
                    break;
                } 
            }
            manualResetEvent4Stop.Set();
        }
        public abstract void Process(long[] ids);
    }
}
