﻿using System;
using System.Collections.Generic;
using System.Text;
using log4net;
using System.IO;
using log4net.Repository.Hierarchy;


namespace Pz.API.QH360.SDK.Common
{
    /// <summary>
    /// 定义日志等级枚举
    /// </summary>
    internal enum LogLevel
    {
        OFF = 0,
        FATAL = 1,
        ERROR = 2,
        WARN = 3,
        INFO = 4,
        DEBUG = 5,
        ALL = 6
    }
    internal class LogHelper
    {
        public static ILog SetConfig(LogLevel level, string aPath, string aAppenderName)
        {
            log4net.Repository.ILoggerRepository aRep = LogManager.CreateRepository(Guid.NewGuid().ToString().Replace("-", string.Empty));
            log4net.Appender.RollingFileAppender aAppender = new log4net.Appender.RollingFileAppender();
            aAppender.Name = aAppenderName;
            string aFileName = string.IsNullOrEmpty(aPath) ? string.Format("log\\{0}\\", aAppenderName) : string.Format("log\\{0}\\{1}\\", aPath, aAppenderName);
            aAppender.File = Path.Combine(AppDomain.CurrentDomain.BaseDirectory, "log\\" + aAppenderName + "\\");
            aAppender.AppendToFile = true;
            aAppender.MaxSizeRollBackups = -1;
            aAppender.MaximumFileSize = "2MB";
            aAppender.RollingStyle = log4net.Appender.RollingFileAppender.RollingMode.Date;
            aAppender.DatePattern = "yyyy-MM-dd HH\".log\"";
            aAppender.StaticLogFileName = false;
            aAppender.LockingModel = new log4net.Appender.FileAppender.MinimalLock();
            aAppender.Layout = new log4net.Layout.PatternLayout("%date %-5p  - %m%n");
            aAppender.ActivateOptions();
            Hierarchy aHierarchy = (Hierarchy)LogManager.GetRepository(aRep.Name);
            aHierarchy.Root.Level = aHierarchy.LevelMap[level.ToString()];
            log4net.Config.BasicConfigurator.Configure(aRep, aAppender);
            aHierarchy.Configured = true;
            return LogManager.GetLogger(aRep.Name, aAppenderName);
        }

        private static ILog internalLog;
        public static ILog InternalLog
        {
            get
            {
                if (internalLog==null)
                    //internalLog = MyLog.APIQH360.Print;
                    internalLog = SetConfig(LogLevel.ERROR, "", "QH360Log");
                return internalLog;
            }
        }
    }
}
