﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Security.Cryptography;
using System.IO;

namespace Pz.API.QH360.SDK.Common
{
    /// <summary>
    /// 加解密类
    /// </summary>
    public class SymmetricMethod
    {
        /// <summary>
        /// 
        /// </summary>
        /// <param name="eCode"></param>
        /// <param name="sourceStr">Source String</param>
        /// <returns>MD5 hashString</returns>
        public static string GetMD5Str(Encoding eCode, string sourceStr)
        {
            byte[] bytes = eCode.GetBytes(sourceStr);
            byte[] md5Bytes = MD5.Create().ComputeHash(bytes);
            StringBuilder sb = new StringBuilder();
            for (int i = 0; i < md5Bytes.Length; i++)
            {
                sb.Append(md5Bytes[i].ToString("x2"));
            }
            return sb.ToString();
        }

        /// <summary>
        /// AES加密
        /// </summary>
        /// <param name="encryptString">待加密字符串</param>
        /// <param name="key">16位密钥</param>
        /// <param name="iv">16位偏移量</param>
        /// <returns></returns>
        public static string EncryptAES(string encryptString, string key, string iv)
        {
            try
            {
                byte[] inputByteArray = Encoding.UTF8.GetBytes(encryptString);
                SymmetricAlgorithm des = Rijndael.Create();
                des.Key = Encoding.ASCII.GetBytes(key.Substring(0, 16));

                des.IV = Encoding.ASCII.GetBytes(iv.Substring(0, 16));
                des.Mode = CipherMode.CBC;
                des.Padding = PaddingMode.Zeros;

                MemoryStream mStream = new MemoryStream();
                CryptoStream cStream = new CryptoStream(mStream, des.CreateEncryptor(), CryptoStreamMode.Write);
                cStream.Write(inputByteArray, 0, inputByteArray.Length);
                cStream.FlushFinalBlock();

                byte[] desBytes = mStream.ToArray();
                StringBuilder sb = new StringBuilder();
                for (int i = 0; i < desBytes.Length; i++)
                {
                    sb.Append(desBytes[i].ToString("x2"));
                }
                mStream.Close();
                cStream.Close();
                return sb.ToString();
            }
            catch (Exception ex)
            {
                return encryptString;
            }

        }

        /// <summary>
        /// AES解密 
        /// </summary>
        /// <param name="decryptString">解密字符串</param>
        /// <param name="key">密钥</param>
        /// <param name="iv">偏移量</param>
        /// <returns></returns>
        public static string DecryptAES(string decryptString, string key, string iv)
        {
            try
            {
                byte[] inputByteArray = strToToHexByte(decryptString);
                SymmetricAlgorithm des = Rijndael.Create();
                des.Key = Encoding.ASCII.GetBytes(key.Substring(0, 16));

                des.IV = Encoding.ASCII.GetBytes(iv.Substring(0, 16));
                des.Padding = PaddingMode.Zeros;
                des.Mode = CipherMode.CBC;
                MemoryStream mStream = new MemoryStream();
                CryptoStream cStream = new CryptoStream(mStream, des.CreateDecryptor(), CryptoStreamMode.Write);
                cStream.Write(inputByteArray, 0, inputByteArray.Length);
                cStream.FlushFinalBlock();
                byte[] desDecryBytes = mStream.ToArray();
                return Encoding.UTF8.GetString(desDecryBytes);
            }
            catch (Exception ex)
            {
                return decryptString;
            }
        }

        /// <summary>
        /// 转16进制字符串
        /// </summary>
        /// <param name="hexString">待转换字符串</param>
        /// <returns></returns>
        private static byte[] strToToHexByte(string hexString)
        {
            try
            {
                hexString = hexString.Replace(" ", "");
                if ((hexString.Length % 2) != 0)
                    hexString += " ";
                byte[] returnBytes = new byte[hexString.Length / 2];
                for (int i = 0; i < returnBytes.Length; i++)
                    returnBytes[i] = Convert.ToByte(hexString.Substring(i * 2, 2), 16);
                return returnBytes;
            }
            catch (Exception ex)
            {
                return null;
            }
        } 

        /// <summary>
        /// 先用MD5 加密，再用AES 对称加密
        /// </summary>
        /// <param name="pToEncrypt">待加密的串</param>
        /// <param name="aKey">秘钥</param>
        /// <returns>经过加密的串</returns>
        public static string MyEncryption(string pToEncrypt, string aKey)
        {
            try
            {
                Encoding aEncoding = Encoding.UTF8;
                string md5 = GetMD5Str(aEncoding, pToEncrypt);
                string key = aKey.Substring(0, 16);
                string iv = aKey.Substring(16, 16);
                return EncryptAES(md5, key, iv);
            }
            catch
            {
                return pToEncrypt;
            }
        }
    }
}
