﻿using Pz.API.QH360.SDK.Common;
using System;
namespace Pz.API.QH360.SDK.Model
{
    /// <summary>
    /// 构造AccessTokenUrl
    /// </summary>
    [Serializable]
    public class MAccessTokenUrl
    {
        /// <summary>
        /// GrantType 的枚举类型
        /// </summary>
        public enum EGrantType
        {
            /// <summary>
            /// 类型为账户code
            /// </summary>
            authorization_code,
            /// <summary>
            /// 类型为刷新token
            /// </summary>
            refresh_token,
        }

        #region 常量值
        /// <summary>
        /// 通过Authorization Code获取Access Token
        /// </summary>
        private const string url_getaccesstoken = MyConfig.OAuthHostUrl + "access_token";
        #endregion

        #region GrantType
        /// <summary>
        /// 此值固定为“authorization_code”
        /// </summary>
        private EGrantType grant_type = EGrantType.authorization_code;
        /// <summary>
        /// 此值固定为“authorization_code”
        /// </summary>
        public EGrantType GrantType
        {
            get { return grant_type; }
            set { grant_type = value; }
        }
        #endregion

        #region Code
        /// <summary>
        /// 通过上面第一步所获得的Authorization Code
        /// </summary>
        private string code = string.Empty;
        /// <summary>
        /// 通过上面第一步所获得的Authorization Code
        /// </summary>
        public string Code
        {
            get { return code; }
            set { code = value; }
        }
        #endregion

        #region ClientId
        /// <summary>
        /// 应用的App Key
        /// </summary>
        private string client_id = string.Empty;
        /// <summary>
        /// 应用的App Key
        /// </summary>
        public string ClientId
        {
            get { return client_id; }
            set { client_id = value; }
        }
        #endregion

        #region ClientSecret
        /// <summary>
        /// 应用的App Secret
        /// </summary>
        private string client_secret = string.Empty;
        /// <summary>
        /// 应用的App Secret
        /// </summary>
        public string ClientSecret
        {
            get { return client_secret; }
            set { client_secret = value; }
        }
        #endregion

        #region RedirectUri
        /// <summary>
        /// redirect_uri所在域名必须与开发者注册应用时所提供的回调地址的域名相匹配
        /// </summary>
        private string redirect_uri = string.Empty;
        /// <summary>
        /// redirect_uri所在域名必须与开发者注册应用时所提供的回调地址的域名相匹配
        /// </summary>
        public string RedirectUri
        {
            get { return redirect_uri; }
            set { redirect_uri = value; }
        }
        #endregion

        #region RefreshToken
        /// <summary>
        /// 用于刷新Access Token用的Refresh Token
        /// </summary>
        private string refresh_token = string.Empty;
        /// <summary>
        /// 用于刷新Access Token用的Refresh Token
        /// </summary>
        public string RefreshToken
        {
            get { return refresh_token; }
            set { refresh_token = value; }
        }
        #endregion

        #region Scope
        /// <summary>
        /// （可选） 
        /// 以空格分隔的权限列表，若不传递此参数，代表请求默认的basic权限。注：通过Refresh Token刷新Access Token时所要求的scope权限范围必须小于等于上次获取Access Token时授予的权限范围
        /// </summary>
        private string scope = string.Empty;
        /// <summary>
        /// （可选） 
        /// 以空格分隔的权限列表，若不传递此参数，代表请求默认的basic权限。注：通过Refresh Token刷新Access Token时所要求的scope权限范围必须小于等于上次获取Access Token时授予的权限范围
        /// </summary>
        public string Scope
        {
            get { return scope; }
            set { scope = value; }
        }
        #endregion

        #region 重写方法
        /// <summary>
        /// 获取字符串
        /// </summary>
        /// <returns></returns>
        public override string ToString()
        {
            return url_getaccesstoken + "?grant_type=" + this.GrantType.ToString()
                + "&client_id=" + this.ClientId + "&client_secret=" + this.ClientSecret
                + (string.IsNullOrEmpty(this.Code) ? string.Empty : "&code=" + this.Code)
                + (string.IsNullOrEmpty(this.RefreshToken) ? string.Empty : "&refresh_token=" + this.RefreshToken)
                + (string.IsNullOrEmpty(this.redirect_uri) ? string.Empty : "&redirect_uri=" + this.RedirectUri)
                + (string.IsNullOrEmpty(this.Scope) ? string.Empty : "&scope=" + this.Scope);
        }
        /// <summary>
        /// 获取HashCode
        /// </summary>
        /// <returns></returns>
        public override int GetHashCode()
        {
            return this.ToString().GetHashCode();
        }
        /// <summary>
        /// 比较
        /// </summary>
        /// <param name="obj"></param>
        /// <returns></returns>
        public override bool Equals(object obj)
        {
            if (typeof(MAuthorizeCodeUrl) != obj.GetType())
                return false;
            if (obj.GetHashCode() == this.GetHashCode())
                return true;
            else
                return false;
        }
        #endregion
    }
}
