﻿using System;
using System.Collections.Generic;
using System.Text;
using Pz.API.QH360.SDK.Common;

namespace Pz.API.QH360.SDK.Model
{
    /// <summary>
    /// 构造获取AccessTokenCode url
    /// </summary>
    [Serializable]
    public class MAuthorizeCodeUrl
    {
        #region 常量值
        /// <summary>
        /// 获取Authorization Code
        /// </summary>
        private const string url_getaccesstoken = MyConfig.OAuthHostUrl + "authorize";
        #endregion

        #region ClientId
        /// <summary>
        /// 必填项
        /// 创建应用时获得的App Key
        /// </summary>
        private string client_id;
        /// <summary>
        /// 必填项
        /// 创建应用时获得的App Key
        /// </summary>
        public string ClientId
        {
            get { return client_id; }
            set { client_id = value; }
        }
        #endregion

        #region ResponseType
        /// <summary>
        /// 必填项
        /// 此值固定为“code”
        /// </summary>
        private string response_type = "code";
        /// <summary>
        /// 必填项
        /// 此值固定为“code”
        /// </summary>
        public string ResponseType
        {
            get { return response_type; }
            set { response_type = value; }
        }
        #endregion

        #region RedirectUri
        /// <summary>
        /// （可选） 
        /// 授权后要回调的URI，即接收Authorization Code的URI, 其值可以是“oob”。 非“oob”值的redirect_uri所在域名必须与开发者注册应用时所提供的回调地址的域名相匹配
        /// </summary>
        private string redirect_uri = string.Empty;
        /// <summary>
        /// （可选） 
        /// 授权后要回调的URI，即接收Authorization Code的URI, 其值可以是“oob”。 非“oob”值的redirect_uri所在域名必须与开发者注册应用时所提供的回调地址的域名相匹配
        /// </summary>
        public string RedirectUri
        {
            get { return redirect_uri; }
            set { redirect_uri = value; }
        }
        #endregion

        #region Scope
        /// <summary>
        /// （可选） 
        /// 以空格分隔的权限列表，若不传递此参数，代表请求默认的basic权限。（目前只有basic权限）
        /// </summary>
        private string scope = string.Empty;
        /// <summary>
        /// （可选） 
        /// 以空格分隔的权限列表，若不传递此参数，代表请求默认的basic权限。（目前只有basic权限）
        /// </summary>
        public string Scope
        {
            get { return scope; }
            set { scope = value; }
        }
        #endregion

        #region State
        /// <summary>
        /// （可选）
        /// 用于保持请求和回调的状态，授权服务器在回调时（重定向用户浏览器到“redirect_uri”时），会在Query Parameter中原样回传该参数
        /// </summary>
        private string state = string.Empty;
        /// <summary>
        /// （可选）
        /// 用于保持请求和回调的状态，授权服务器在回调时（重定向用户浏览器到“redirect_uri”时），会在Query Parameter中原样回传该参数
        /// </summary>
        public string State
        {
            get { return state; }
            set { state = value; }
        }
        #endregion

        #region OauthVersion
        /// <summary>
        /// （可选）
        /// 版本号，如果填写必须为1.0
        /// </summary>
        private string oauth_version = string.Empty;
        /// <summary>
        /// （可选）
        /// 版本号，如果填写必须为1.0
        /// </summary>
        public string OauthVersion
        {
            get { return oauth_version; }
            set { oauth_version = value; }
        }
        #endregion

        #region Display
        /// <summary>
        /// （可选）
        /// 登录和授权页面的展现样式，360桌面应用请传递“desktop”，默认为“default”或空。
        /// </summary>
        private string display = string.Empty;
        /// <summary>
        /// （可选）
        /// 登录和授权页面的展现样式，360桌面应用请传递“desktop”，默认为“default”或空。
        /// </summary>
        public string Display
        {
            get { return display; }
            set { display = value; }
        }
        #endregion

        #region Relogin
        /// <summary>
        /// （可选）
        /// 仅在实现"使用360账号登陆"功能时才需要传递。当浏览器有360cookie时，传递relogin可展示“当前账号登陆确认页”；relogin值请传递公司域名，如www.360.cn可传递"relogin=360.cn"
        /// </summary>
        private string relogin = string.Empty;
        /// <summary>
        /// （可选）
        /// 仅在实现"使用360账号登陆"功能时才需要传递。当浏览器有360cookie时，传递relogin可展示“当前账号登陆确认页”；relogin值请传递公司域名，如www.360.cn可传递"relogin=360.cn"
        /// </summary>
        public string Relogin
        {
            get { return relogin; }
            set { relogin = value; }
        }
        #endregion

        #region 重写方法
        /// <summary>
        /// 获取GetAccessToken 路径
        /// </summary>
        /// <returns></returns>
        public override string ToString()
        {
            return url_getaccesstoken + "?client_id=" + this.client_id + "&response_type=" + this.ResponseType
                + (string.IsNullOrEmpty(this.redirect_uri) ? string.Empty : "&redirect_uri=" + this.RedirectUri)
                + (string.IsNullOrEmpty(this.scope) ? string.Empty : "&scope=" + this.Scope)
                + (string.IsNullOrEmpty(this.state) ? string.Empty : "state=" + this.State)
                + (string.IsNullOrEmpty(this.oauth_version) ? string.Empty : "&oauth_version=" + this.OauthVersion)
                + (string.IsNullOrEmpty(this.display) ? string.Empty : "&display=" + this.Display)
                + (string.IsNullOrEmpty(this.relogin) ? string.Empty : "&relogin=" + this.Relogin);
        }
        /// <summary>
        /// 获取HashCode
        /// </summary>
        /// <returns></returns>
        public override int GetHashCode()
        {
            return this.ToString().GetHashCode();
        }
        /// <summary>
        /// 比较
        /// </summary>
        /// <param name="obj"></param>
        /// <returns></returns>
        public override bool Equals(object obj)
        {
            if (typeof(MAuthorizeCodeUrl) != obj.GetType())
                return false;
            if (obj.GetHashCode() == this.GetHashCode())
                return true;
            else
                return false;
        }
        #endregion

    }
}
