﻿using System;

namespace Pz.API.QH360.SDK.Model
{
    /// <summary>
    /// 关键词实体
    /// </summary>
    [Serializable]
    public class QH360MKeyword : ApiResult, IComparable<QH360MKeyword>
    {
        /// <summary>
        /// id
        /// </summary>
        public long id;


        /// <summary>
        /// 组id
        /// </summary>
        public long groupId;


        /// <summary>
        /// 关键词
        /// </summary>
        public string word;


        /// <summary>
        /// 关键词价格 
        /// </summary>
        public double price;


        /// <summary>
        /// exact表示精确，phrase表示短语，broad 表示广泛匹配
        /// </summary>
        public string matchType;

        /// <summary>
        /// 链接地址
        /// </summary>
        public string destinationUrl;

        /// <summary>
        /// 状态
        /// </summary>
        public string status { get; set; }

        /// <summary>
        /// 质量度
        /// </summary>
        public int qualityScore { get; set; }

        /// <summary>
        /// 关键词创建时间
        /// </summary>
        public string addTime { get; set; }

        /// <summary>
        /// 推广创意最后更新时间 
        /// </summary>
        public string updateTime;

        /// <summary>
        /// 按照匹配方式排序
        /// </summary>
        /// <param name="otherKeyword"></param>
        /// <returns></returns>
        public int CompareTo(QH360MKeyword otherKeyword)
        {
            string thisMatchType = this.matchType.ToLower();
            string otherMatchType = otherKeyword.matchType;
            return thisMatchType.CompareTo(otherMatchType);
        }

        /// <summary>
        /// 在批量添加关键词时，方便有json格式转化
        /// json格式关键词匹配方式：exact表示精确，phrase表示
        ///短语，broad 表示广泛匹配。每个关键词包含三个属性，
        ///其中price和url可以不填，price如果不填，默认为广
        ///告组出价，url不填则使用创意的链接网址。如 
        ///{” phrase”:[{“word“:”短语关键词
        ///1”,”price”:0.1,”url”:http://www.360.cn}],”exact”:
        ///[ {“word“:”关键词
        ///2”,”price”:0.1,”url”:http://www.360.cn}]}
        /// </summary>
        public class Keywords
        {
            public KeywordItem[] broad { get; set; }
            public KeywordItem[] exact { get; set; }
            public KeywordItem[] phrase { get; set; }
        }

        /// <summary>
        /// 一条关键词
        /// </summary>
        public class KeywordItem
        {
            /// <summary>
            /// 关键词
            /// </summary>
            public string word;
            /// <summary>
            /// 关键词出价
            /// </summary>
            public double price;
            /// <summary>
            /// 链接地址
            /// </summary>
            public string url;
        }
    }
}
