﻿using System;
using System.Collections.Generic;
using System.IO;
using System.Net;
using System.Net.Security;
using System.Security.Cryptography.X509Certificates;
using System.Threading;
using System.Web;
using log4net;
using Pz.API.QH360.SDK.API;
using Pz.API.QH360.SDK.Common;
using Pz.API.QH360.SDK.Enum;



namespace Pz.API.QH360.SDK.Model
{
    /// <summary>
    /// 获取httpRequset
    /// </summary>
    internal class MyHttpRequest
    {

        ILog log = LogHelper.InternalLog;

        /// <summary>
        /// 定义请求的代理
        /// </summary>
        internal IWebProxy webProxy { get; set; }
        /// <summary>
        /// 设置头部信息
        /// </summary>
        internal PostHeaders authHeader { get; set; }
        /// <summary>
        /// 设置超时时间
        /// </summary>
        internal int webTimeOut { get; set; }
        /// <summary>
        /// 设置服务类型
        /// </summary>
        internal ServiceType serviceType { get; set; }
        /// <summary>
        /// 设置方法类型
        /// </summary>
        internal MethodType methodType { get; set; }
        /// <summary>
        /// 获取数据
        /// </summary>
        internal string data { get; private set; }
        /// <summary>
        /// 当前状态
        /// </summary>
        internal ResultStatus status { get; private set; }

        /// <summary>
        /// 响应Code
        /// </summary>
        internal int ResultCode { get; private set; }
        /// <summary>
        /// 本次请求消耗的配额
        /// </summary>
        internal int QuotaCost { get; private set; }
        /// <summary>
        /// 当前API Key剩余的配额
        /// </summary>
        internal int QuotaRemain { get; private set; }

        private volatile int exceptionCount = 0; //这个用于记录网络异常的次数,当小于最大异常次数时会重新继续获取,直到个数超过
        /// <summary>
        /// 允许的最大异常次数
        /// </summary>
        private int maxExceptionCount = 10;
        public int MaxExceptionCount { get { return maxExceptionCount; } set { maxExceptionCount = value; } }

        private string postParStr = string.Empty;

        private volatile bool isWebExpetionErr = true;
        /// <summary>
        /// 获取提交数据
        /// </summary>
        /// <param name="parameters"></param>
        /// <returns></returns>
        private byte[] getPostData(List<MyPostParameter> parameters)
        {
            string data = "format=" + MyConfig.MyFormat.ToString();
            if (null != parameters)
            {
                foreach (MyPostParameter p in parameters)
                {
                    string name = p.Name;
                    object objValue = p.Value;
                    string value = "";
                    if (objValue != null)
                    {
                        value = objValue.ToString();
                        value = HttpUtility.UrlEncode(value, MyConfig.PostEncoding);
                    }
                    data += "&" + name + "=" + value;
                }
            }
            string info = "ServiceType:" + this.serviceType.ToString() + "\t\n"
                          + "MethodType:" + this.methodType.ToString() + "\t\n"
                          + "Parameters:" + data;
            this.postParStr = info;
            log.Info(info);
            return MyConfig.PostEncoding.GetBytes(data);
        }

        /// <summary>
        /// 获取页面返回值
        /// </summary>
        /// <param name="p"></param>
        /// <param name="aHtml"></param>
        /// <returns></returns>
        internal void postResponse(List<MyPostParameter> parameters)
        {
            byte[] data = this.getPostData(parameters);
            this.postResponse(data);
        }

        internal void postResponse(List<MyPostParameter> parameters,string url)
        {
            byte[] data = this.getPostData(parameters);
            this.postResponse(data,url);
        }


        private void postResponse(byte[] postData)
        {
            string url = string.Format(MyConfig.APIHostUrl, MyConfig.Version, serviceType.ToString(), methodType.ToString());
            postResponse(postData,url);
        }
        /// <summary>
        /// 获取页面返回值
        /// </summary>
        /// <param name="p"></param>
        /// <param name="aHtml"></param>
        /// <returns></returns>
        private void postResponse(byte[] postData,string url)
        {
            if (null == this.authHeader)
                throw new Exception("Header Can't be null");
            this.status = ResultStatus.Succeed;
           // string url = string.Format(MyConfig.APIHostUrl, MyConfig.Version, serviceType.ToString(), methodType.ToString());
            log.Info("apiUrl:" + url + "\n");
            System.GC.Collect();
            ServicePointManager.DefaultConnectionLimit = 500;
            HttpWebRequest aHttpWebRequest = null;
            try
            {
                if (url.StartsWith("https", StringComparison.OrdinalIgnoreCase))
                    ServicePointManager.ServerCertificateValidationCallback = new RemoteCertificateValidationCallback(CheckValidationResult);
                aHttpWebRequest = WebRequest.Create(url) as HttpWebRequest;
                if (this.webTimeOut > 0)
                    aHttpWebRequest.Timeout = this.webTimeOut;
                aHttpWebRequest.Proxy = webProxy;
                aHttpWebRequest.CookieContainer = new CookieContainer();
                aHttpWebRequest.Method = "POST";
                aHttpWebRequest.Headers.Set("Accept-Encoding", "gzip");
                aHttpWebRequest.KeepAlive = false;
                aHttpWebRequest.ProtocolVersion = HttpVersion.Version10;
                aHttpWebRequest.UserAgent = "curl/7.28.1";
                aHttpWebRequest.Accept = "*/*";
                aHttpWebRequest.Headers.Add("apiKey", this.authHeader.apiKey);
                aHttpWebRequest.Headers.Add("accessToken", this.authHeader.accessToken);
                aHttpWebRequest.Headers.Add("serveToken", this.authHeader.serveToken);
                if (!string.IsNullOrEmpty(this.authHeader.appId))
                aHttpWebRequest.Headers.Add("appid", this.authHeader.appId);
                if (!string.IsNullOrEmpty(this.authHeader.sessionToken))
                aHttpWebRequest.Headers.Add("sessionToken", this.authHeader.sessionToken);
                aHttpWebRequest.ContentType = "application/x-www-form-urlencoded";
                aHttpWebRequest.ServicePoint.Expect100Continue = false;
                aHttpWebRequest.ServicePoint.ConnectionLimit = int.MaxValue;
                if (null == postData)
                    postData = new byte[0];
                aHttpWebRequest.ContentLength = postData.Length;
                using (Stream aStream = aHttpWebRequest.GetRequestStream())
                {
                    aStream.Write(postData, 0, postData.Length);
                }
                using (HttpWebResponse aHttpWebResponse = (HttpWebResponse)aHttpWebRequest.GetResponse())
                {
                    WebHeaderCollection resHeadCol = aHttpWebResponse.Headers;
                    int resultCode = 0;
                    int quotaCost = 0;
                    int quotaRemain = 0;
                    foreach (string key in resHeadCol.AllKeys)
                    {
                        if (key.Equals("resultCode"))
                            int.TryParse(resHeadCol[key], out resultCode);
                        if (key.Equals("quotaCost"))
                            int.TryParse(resHeadCol[key], out quotaCost);
                        if (key.Equals("quotaRemain"))
                            int.TryParse(resHeadCol[key], out quotaRemain);
                    }
                    this.ResultCode = resultCode;
                    this.QuotaCost = quotaCost;
                    this.QuotaRemain = quotaRemain;
                    using (StreamReader reader = new StreamReader(aHttpWebResponse.GetResponseStream(), MyConfig.MyEncoding))
                    {
                        this.data = reader.ReadToEnd();
                    }
                    log.Info("ApiResData:" + this.data + "\n");
                    if (!string.IsNullOrEmpty(this.data))
                    {
                        //假如有了Too many requests in one second错误,递归重试
                        ApiResult apiResult = Json.Deserialize<ApiResult>(this.data);
                        if (apiResult != null && apiResult.failures != null && apiResult.failures.Length > 0)
                        {
                            //打印有失败的日志
                            log.Error(this.data);
                            Failure fail = apiResult.failures[0];
                            if (fail.code.Equals(80402))
                            {
                                exceptionCount++;
                                log.Error("第" + exceptionCount.ToString() + "失败发生了请求太过频繁，等待30s后重试的错误");
                                if (exceptionCount < maxExceptionCount)
                                {
                                    Thread.Sleep(35000); //等待30重试
                                    postResponse(postData);  //进行递归重新获取,直到超过最大限度
                                }
                            }
                        }
                    }
                    else
                        this.status = ResultStatus.DataIsEmpty;
                }
                isWebExpetionErr = false;
            }
            catch (WebException e)
            {
                exceptionCount++;
                log.Error("第" + exceptionCount.ToString() + "失败" + e.Message + " WebExceptionStatus:" + e.Status.ToString());
                if (exceptionCount < maxExceptionCount)
                {
                    //递归前要释放连接
                    if (aHttpWebRequest != null)
                    {
                        aHttpWebRequest.Abort();
                        aHttpWebRequest = null;
                    }
                    postResponse(postData);  //进行递归重新获取,直到超过最大限度
                    if (!isWebExpetionErr)
                        return;
                }
                this.status = ResultStatus.Fail;
                if (null != e.Response)
                {
                    using (StreamReader reader = new StreamReader(e.Response.GetResponseStream(), MyConfig.MyEncoding))
                    {
                        this.data = reader.ReadToEnd();
                    }
                    log.Error("ApiResData:" + this.data + "\n");
                }
                log.Error(e.Message, e);
                throw e;
            }
            finally
            {
                if (aHttpWebRequest != null)
                {
                    ServicePoint point = aHttpWebRequest.ServicePoint;
                    string connectionsGroupName = aHttpWebRequest.ConnectionGroupName;
                    if (point != null)
                    {
                        if (point.CurrentConnections > 10)
                            point.CloseConnectionGroup(connectionsGroupName);
                    }
                    aHttpWebRequest.Abort();
                    aHttpWebRequest = null;
                }
            }
        }
        /// <summary>
        /// 构造函数
        /// </summary>
        internal MyHttpRequest()
        {
            int keepAliveTime = 10 * 1000;
            int keepAliveInterval = 300;
            System.Net.ServicePointManager.SetTcpKeepAlive(true, keepAliveTime, keepAliveInterval);
        }
        internal MyHttpRequest(APIParent aPIParent)
            : this()
        {
            this.authHeader = aPIParent.AuthHeader;
            this.webTimeOut = aPIParent.WebTimeOut;
            this.webProxy = aPIParent.WebProxy;
            this.serviceType = aPIParent.ServiceType;
        }

        /// <summary>
        /// 返回结果数据
        /// </summary>
        /// <param name="res"></param>
        internal void GetRes(ref Result res)
        {
            res.Status = this.status;
            res.ResultCode = this.ResultCode;
            res.QuotaCost = this.QuotaCost;
            res.QuotaRemain = this.QuotaRemain;
            QhApiGlobal.CurruntResult = res;
        }

        private bool CheckValidationResult(object sender, X509Certificate certificate, X509Chain chain, SslPolicyErrors errors)
        {
            //总是接受          
            return true;
        }
    }
    /// <summary>
    /// 提交的数据
    /// </summary>
    internal class MyPostParameter
    {
        /// <summary>
        /// 实例参数对象
        /// </summary>
        /// <param name="name"></param>
        /// <param name="value"></param>
        public MyPostParameter(string name, object value)
        {
            this.Name = name;
            this.Value = value;
        }
        /// <summary>
        /// 参数名称
        /// </summary>
        public string Name { get; private set; }
        /// <summary>
        /// 值
        /// </summary>
        public object Value { get; private set; }

        /// <summary>
        /// 获取String
        /// </summary>
        /// <returns></returns>
        public override string ToString()
        {
            return this.Name + "=" + this.Value;
        }
        /// <summary>
        /// 获取HashCode
        /// </summary>
        /// <returns></returns>
        public override int GetHashCode()
        {
            return this.ToString().GetHashCode();
        }
        /// <summary>
        /// 判断对象是否相同
        /// </summary>
        /// <param name="obj"></param>
        /// <returns></returns>
        public override bool Equals(object obj)
        {
            if (typeof(MyPostParameter) != obj.GetType())
                return false;
            return (this.GetHashCode() == obj.GetHashCode());
        }
    }
}
