﻿using System;

namespace Pz.API.QH360.SDK.Model
{
    /// <summary>
    /// 360api请求的头部信息
    /// </summary>
    [Serializable]
    public class PostHeaders
    {
        /// <summary>
        /// 注册时分配到的 api key，该参数作为HTTP HEAD字段传递。 
        /// </summary>
        public string apiKey { get; set; }

        /// <summary>
        /// 注册时分配到的apiSecretID
        /// </summary>
        public string apiSecretID { get; set; }
        /// <summary>
        /// OAuth 身份认证通过后平台分配的
        /// token，作为操作对应广告帐户的凭证，该参数作为HTTP HEAD字段传递。 
        /// </summary>
        public string accessToken { get; set; }
        /// <summary>
        /// 处理凭证，目前为时间戳，系统时间的毫秒值,同个应用的不同 api 请求的 token 应该是递增的, 用于防replay攻击，该参数作为HTTP HEAD字段传递。 
        /// </summary>
        public string serveToken { get { return DateTime.Now.Ticks.ToString(); } }
        /// <summary>
        /// 用于区别同一个开发者的多个应用（不同 版本的同一个应用appid也不同）
        /// appid格式要求：最大长度为15，允许字符为英文字母、数字0-9和半角小数点.的组合，建议采用应用名称+版本号的形式，如xxxx.1.10.001 对于在点睛上线发布的应用，一律要求请
        ///求中提供正确的appid，对于开发者的其他自用应用该参数值可以为空； 该参数作为HTTP HEAD字段传递。
        /// </summary>
        public string appId { get; set; }

        /// <summary>
        /// 用户标识。该参数作为HTTP HEAD字段传递。参数获取方式见接口clientLogin
        /// </summary>
        public string sessionToken { get; set; }
    }
}
